"""Top-level package for aws_pcluster_bootstrap_helpers."""

__author__ = """Jillian Rowe"""
__email__ = "jillian@dabbleofdevops.com"
__version__ = "0.1.0"

from . import _version
__version__ = _version.get_versions()['version']



from pcluster.cli.commands.configure.easyconfig import _get_subnets, _get_vpcs_and_subnets, _create_vpc_parameters
import os

from prefect import flow, task
import re

import inquirer

from rich.prompt import Prompt
from rich.panel import Panel

from textual.app import App
from textual.reactive import Reactive
from textual.widget import Widget
from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

logger = setup_logger('configure')


def get_network_data(region: str = "us-east-1"):
    os.environ['AWS_DEFAULT_REGION'] = region
    network_data = _get_vpcs_and_subnets()
    return network_data


def configure(
    region: str = "us-east-1"
):
    network_data = get_network_data(region)
    vpcs = network_data['vpc_list']
    vpc_subnets = network_data['vpc_subnets']
    vpc_choices = list(map(lambda x: f"{x['id']} {x['name']}", vpcs))
    vpc_choices_options = list(
        map(lambda x: dict(label=f"{x['id']} {x['name']}", data=x), vpcs)
    )
    questions = [
        inquirer.List('vpc_id',
                      message="Choose a VPC",
                      choices=vpc_choices,
                      ),
    ]
    answers = inquirer.prompt(questions)
    vpc_choice = list(filter(lambda x: x['label'] == answers['vpc_id'], vpc_choices_options))[0]
    vpc_id = vpc_choice['data']['id']
    subnets = vpc_subnets[vpc_id]

    subnet_choices = list(map(lambda x: f"{x['id']} {x['name']}", subnets))
    subnet_choices_options = list(
        map(lambda x: dict(label=f"{x['id']} {x['name']}", data=x), subnets)
    )
    questions = [
        inquirer.List('head_subnet',
                      message="Choose a subnet for the head node",
                      choices=subnet_choices,
                      ),
        inquirer.List('compute_subnet',
                      message="Choose a subnet for the compute nodes",
                      choices=subnet_choices,
                      ),
    ]
    answers = inquirer.prompt(questions)
    head_subnet = list(filter(lambda x: x['label'] == answers['head_subnet'], subnet_choices_options))[0]
    compute_subnet = list(filter(lambda x: x['label'] == answers['compute_subnet'], subnet_choices_options))[0]
    head_subnet_id = head_subnet['data']['id']
    compute_subnet_id = compute_subnet['data']['id']

    subnet_ids = [head_subnet_id, compute_subnet_id]
    if head_subnet_id == compute_subnet_id:
        subnet_ids = [head_subnet_id]

    network_data = dict(
        vpc_id=vpc_id,
        subnet_ids=subnet_ids
    )
    print(network_data)
    return network_data



import os
import pathlib
from prefect import flow, task
from prefect_shell import shell_run_command
import time
import tempfile
import json

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger
from pcluster.api.controllers.cluster_operations_controller import list_clusters

from pcluster import utils

PCLUSTER_VERSION = utils.get_installed_version()

logger = setup_logger("create-cluster")

CREATE_IN_PROGRESS = "CREATE_IN_PROGRESS"
CREATE_FAILED = "CREATE_FAILED"
CREATE_COMPLETE = "CREATE_COMPLETE"
DELETE_IN_PROGRESS = "DELETE_IN_PROGRESS"
DELETE_FAILED = "DELETE_FAILED"
DELETE_COMPLETE = "DELETE_COMPLETE"


def parse_json_status(content, file):
    f = open(file)
    try:
        data = json.load(f)
    except Exception as e:
        logger.warn(f"Error reading in pcluster create file: {file}: {e}")
        raise Exception(e)

    cluster_status = data["clusterStatus"]
    logger.info(f"Pcluster create status: {cluster_status}")
    if "FAILED" in cluster_status:
        raise Exception(f"Cluster create failed: {cluster_status}")
    elif cluster_status == "CREATE_IN_PROGRESS":
        return True
    elif "PROGRESS" in cluster_status:
        return True
    elif "COMPLETE" not in cluster_status and "FAIL" not in cluster_status:
        return True
    elif cluster_status == "CREATE_COMPLETE":
        return False
    else:
        raise Exception(f"Cluster status not compatible with bootstrap: {cluster_status}")


def watch_create_cluster(cluster_name: str, region="us-east-1"):
    create_in_process = True
    n = 1
    while create_in_process:
        with tempfile.NamedTemporaryFile(suffix=".json") as tmpfile:
            logger.info(
                f"Pcluster: {cluster_name}, Region: {region}, N: {1} Data file: {tmpfile.name}"
            )
            contents = shell_run_command(
                command=f"""pcluster \\
                describe-cluster \\
                -n {cluster_name} \\
                -r {region} > {tmpfile.name}""",
                return_all=True,
            )
            create_in_process = parse_json_status(contents, tmpfile.name)
        n = n + 1
        # sleep for 10 minutes
        if create_in_process:
            time.sleep(60)
    return create_in_process


def describe_cluster(cluster_name: str, output_file: str, region="us-east-1"):
    contents = shell_run_command(
        command=f"""pcluster \\
        describe-cluster \\
        -n {cluster_name} \\
        --region {region} > {output_file}""",
        return_all=True,
    )
    create_in_process = parse_json_status(contents, output_file)
    return create_in_process


def create_cluster(cluster_name: str, region: str, config_file: str):
    state = None
    clusters = list_clusters(region=region)
    found = False
    if len(clusters.clusters):
        for cluster in clusters.clusters:
            if 'cluster_name' in cluster:
                if cluster_name == cluster['cluster_name']:
                    found = True
    if found:
        logger.info(
            f"""Cluster {cluster_name} already found.
             If you would like to create a new cluster you must first delete this cluster.
             """
        )
        state = True
    else:
        state = shell_run_command(
            command=f"""pcluster create-cluster \\
      --rollback-on-failure false \\
      -n {cluster_name} \\
      -r {region} \\
      -c {config_file}
    """,
            return_all=True,
        )
    return


@flow
def watch_cluster_create_flow(
    cluster_name: str,
    config_file: pathlib.Path,
    region: str = "us-east-1",
):
    create_cluster(cluster_name=cluster_name, region=region, config_file=str(config_file))
    watch_create_cluster(cluster_name=cluster_name, region=region)
    return True


@flow
def create_cluster_flow(
    cluster_name: str,
    config_file: pathlib.Path,
    region: str = "us-east-1",
    pcluster_version: str = "3.2",
):
    if pcluster_version not in PCLUSTER_VERSION:
        w = f"""Mismatch between specified pcluster version and installed
        Specified: {pcluster_version}, Installed: {PCLUSTER_VERSION}
        """
        raise ValueError(w)
    create_cluster(cluster_name=cluster_name, region=region, config_file=str(config_file))
    return True



import logging
from pathlib import Path
from typing import Union

import os
from colorlog import ColoredFormatter

seperator = "----------------------------------------------------\n"

levels = {
    "critical": logging.CRITICAL,
    "error": logging.ERROR,
    "warn": logging.WARNING,
    "warning": logging.WARNING,
    "info": logging.INFO,
    "debug": logging.DEBUG,
}


def setup_logger(
    logger_name: str = "[pcluster helpers]",
    log_level: str = os.environ.get("LOG_LEVEL", "INFO"),
    **kwargs,
) -> logging.getLogger:
    """
    _summary_

    Parameters
    ----------
    log_file : Union[str, Path]
        _description_
    log_level : str, optional
        _description_, by default "INFO"
    logger_name : str, optional
        _description_, by default "[Slide-seq Flowcell Alignment Workflow]"

    Returns
    -------
    logging.getLogger
        _description_
    """
    colored_log_format = "%(asctime)s%(reset)s | %(name)s%(reset)s | %(filename)s:%(lineno)d%(reset)s | %(log_color)s%(levelname)-8s%(reset)s | %(message)s"
    file_log_format = "%(asctime)s | %(name)s | %(filename)s:%(lineno)d | %(levelname)-8s | %(message)s"

    color_formatter = ColoredFormatter(
        colored_log_format,
        datefmt=None,
        reset=True,
        log_colors={
            "DEBUG": "cyan",
            "INFO": "green",
            "WARNING": "yellow",
            "ERROR": "red",
            "CRITICAL": "red",
        },
    )

    logger = logging.getLogger(logger_name)
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(color_formatter)

    logger.addHandler(stream_handler)
    logger.setLevel(log_level)

    return logger



import os
from typing import Dict, Any, List, Optional
import pathlib
from prefect import flow, task
from prefect_shell import shell_run_command
import time
import tempfile
import json
from pcluster.api.controllers.image_operations_controller import describe_image
from pcluster.api.models import DescribeImageResponseContent

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

from pcluster import utils

PCLUSTER_VERSION = utils.get_installed_version()

logger = setup_logger("build-ami")

BUILD_IN_PROGRESS = "BUILD_IN_PROGRESS"
BUILD_FAILED = "BUILD_FAILED"
BUILD_COMPLETE = "BUILD_COMPLETE"
DELETE_IN_PROGRESS = "DELETE_IN_PROGRESS"
DELETE_FAILED = "DELETE_FAILED"
DELETE_COMPLETE = "DELETE_COMPLETE"


def parse_image_status(data: DescribeImageResponseContent):
    image_status = data.image_build_status
    logger.info(f"Pcluster build status: {image_status}")
    if image_status == "BUILD_FAILED":
        raise Exception(f"Image build failed: {image_status}")
    elif "FAILED" in image_status:
        raise Exception(f"Image build failed: {image_status}")
    elif image_status == "BUILD_IN_PROGRESS":
        return True
    elif "PROGRESS" in image_status:
        return True
    elif "COMPLETE" not in image_status and "FAIL" not in image_status:
        return True
    elif "COMPLETE" in image_status:
        return False
    elif image_status == "BUILD_COMPLETE":
        return False
    else:
        raise Exception(f"Image status not compatible with bootstrap: {image_status}")


def build_in_progress(image_id: str, region="us-east-1"):
    build_in_process = True
    n = 1
    while build_in_process:
        build_data = describe_image(image_id=image_id)
        build_in_process = parse_image_status(build_data)
        n = n + 1
        # sleep for 10 minutes
        if build_in_process:
            time.sleep(600)


def build_complete(image_id: str, output_file: str, region="us-east-1") -> DescribeImageResponseContent:
    build_data = describe_image(image_id=image_id)
    return build_data


def start_build(image_id: str, region: str, config_file: str):
    shell_run_command(
        command=f"""pcluster build-image \\
  --image-id {image_id} \\
  -r {region} \\
  -c {config_file}
""",
        return_all=True,
    )
    return


@flow
def build_ami_flow(
    image_id: str,
    output_file: pathlib.Path,
    config_file: pathlib.Path,
    region: str = "us-east-1",
    pcluster_version: str = "3.2",
):
    if pcluster_version not in PCLUSTER_VERSION:
        w = f"""Mismatch between specified pcluster version and installed
        Specified: {pcluster_version}, Installed: {PCLUSTER_VERSION}
        """
        raise ValueError(w)
    return True


@flow
def watch_ami_build_flow(
    image_id: str,
    output_file: pathlib.Path,
    config_file: pathlib.Path,
    region: str = "us-east-1"
):
    output_file = str(output_file)
    config_file = str(config_file)
    start_build(image_id=image_id, region=region, config_file=config_file, )
    build_in_progress(image_id=image_id, region=region)
    build_data = build_complete(
        image_id=image_id, region=region, output_file=output_file
    )
    return build_data


@flow
def watch_ami_flow(image_id: str, output_file: pathlib.Path, region: str = "us-east-1"):
    build_in_progress(image_id=image_id, region=region)
    build_data = build_complete(
        image_id=image_id, region=region, output_file=output_file
    )


def main(image_id: str, output_file: pathlib.Path, region: str = "us-east-1"):
    output_file = str(output_file)
    flow_state = watch_ami_build_flow(
        image_id=image_id, region=region, output_file=output_file
    )
    return flow_state
